#include <SimpleModbusRtu_mega2560.h>

// comfileHMI 통신용
#define     comm_baudRate   57600   // 통신속도
#define     comm_slave      1       // 슬레이브 주소

#define     THRT1_ADDR      1       // ML_THRT1 주소

#define     isP38on         (modbusCoil[0] & 0x01)
#define     isP39on         (modbusCoil[0] & 0x02)
#define     isP40on         (modbusCoil[0] & 0x04)
#define     isP41on         (modbusCoil[0] & 0x08)

typedef     enum{P38=38,P39,P40,P41}  PORT_ALIAS;

//모드버스 메모리 영역
unsigned char modbusCoil[10];
unsigned short modbusRegister[10];

unsigned char data[8] = {0x00,0x03,0x01,0x90,0x00,0x01,0x00,0x00};  // index0 : slave_addr, index 6,7 : CRC16

float getTemp();
void setup() {

  pinMode(P38,OUTPUT);
  pinMode(P39,OUTPUT);
  pinMode(P40,OUTPUT);
  pinMode(P41,OUTPUT);
  
  usart1init(comm_baudRate);
  startSimpleModbus(comm_slave,modbusRegister,modbusCoil);
  Serial3.begin(57600);
  while(!Serial3);
}

void loop() {
  float temp = getTemp();

  if((-100.0f <= temp) && (temp <= 500.0f))
  {
    *(float*)&modbusRegister[0] = temp;
  }

  digitalWrite(P38,isP38on);
  digitalWrite(P39,isP39on);
  digitalWrite(P40,isP40on);
  digitalWrite(P41,isP41on);
  
  delay(100);
}

float getTemp()
{
  int i;
  
  // 온도값 요청.
  data[0] = THRT1_ADDR; // 주소설정.
  int crc = getCrc(data,6);
  data[6] = crc>>8;
  data[7] = crc;
  
  while(Serial3.available())  unsigned char dump = Serial3.read();
  _delay_ms(1);
  Serial3.write(data,8);
  Serial3.flush();
  _delay_ms(1);

  for(i=0;i<10;i++)
  {
    if(Serial3.available() >= 7)  break;
    delay(10);
  }

  if(i<10)
  {
    unsigned char received_data[7];
    Serial3.readBytes(received_data,7);
    int tmp_crc = getCrc(received_data,5);
    if((received_data[5] ^ (unsigned char)(tmp_crc>>8)) || (received_data[6] ^ (unsigned char)(tmp_crc)))    return -300.0f;
    return ((received_data[3] << 8) + received_data[4]) / 10.0f;
  }
  else  //error.
  {
    return -200.0f;
  }
}

